<div id="support-future-development-popup" class="modal fade bs-modal-lg draggable-modal" tabindex="-1" role="dialog"
     aria-hidden="true">
    <div class="modal-dialog modal-lg">
        <div class="modal-content" style="background-color: #465568 !important;">

            <div class="modal-header font-white no-border">
                <button type="button" class="close font-white" data-dismiss="modal"
                        aria-hidden="true"></button>
            </div>
            <div class="modal-body">
                <div class="page-body">
                    <h2 class="support-head">Consider Supporting Unmanic</h2>
                    <div class="lock-body">
                        <div class="row">
                            <div class="col-md-5">
                                <div class="pull-left lock-avatar-block">
                                    <img src="/assets/img/logo-lg.png" alt="logo" style="width: 100%;"/>
                                </div>
                            </div>
                            <div class="col-md-7 pull-left">
                                <p class="font-white">
                                    This project is maintained in my spare time.
                                    While I enjoy doing this and I give it to you freely, I would be very grateful if you
                                    considered supporting the project.
                                    <br><br>
                                    Sponsors of Unmanic will receive dedicated support channels and additional content
                                    within the application.
                                    This is my way of showing my appreciation.
                                </p>
                                <button type="button" class="btn red uppercase login-with-patreon">
                                    <i class="socicon-btn socicon-solid socicon-sm socicon-patreon font-white"></i>
                                    Login with Patreon
                                </button>
                            </div>
                        </div>
                    </div>
                </div>
            </div>

            <div class="modal-footer">
                <button type="button" data-dismiss="modal" class="btn btn-default support-future-development-popup-close">
                    Close
                </button>
            </div>
        </div>
    </div>

</div>
